local K = require("game.constants")
local LG = love.graphics
local PI = math.pi

local pairs, ipairs = pairs, ipairs
local match = string.match

local tweener = require("tweener.base")
local easing = require("tweener.easing")

local unit = 16
local arrow = { {8, 7, 10, 7, 10, 9, 8, 9}, {10, 6, 12, 8, 10, 10} }

local tweens = tweener("loopforward") do
  tweens.add(2, { offset = unit / 4 }, easing.outBounce)
  tweens.add(2, { offset = unit / 7 }, easing.inSin)
end

local function render(x, y, scale, kind)
  scale = scale or 1

  local polis, offset = {}, nil

  offset = tweens.getCurrentProperties().offset

  -- displace by offset and save in the polis table
  for _, current in ipairs(arrow) do
    poly = {}
    for i, v in ipairs(current) do
      if i % 2 == 0 then poly[i] = v else poly[i] = v + offset end -- only translate x coordinate
    end
    polis[#polis + 1] = poly
  end

  LG.setBlendMode("additive")
  LG.setColor(255, 255, 255, 212)

  LG.push()
  LG.translate(x + (K.GEM_SIZE * (1 - scale) / 2), y + (K.GEM_SIZE * (1 - scale) / 2))
  LG.scale(K.GEM_SIZE / unit * scale, K.GEM_SIZE / unit * scale)

  local function doArrow(draw)
    if draw then for _, poly in ipairs(polis) do LG.polygon("fill", poly) end end
    LG.translate(unit / 2, unit / 2)
    LG.rotate(PI / 4)
    LG.translate(-unit / 2, -unit / 2)
  end

  for _, direction in pairs{ "r", "4", "d", "3", "l", "1", "u", "2" } do
    doArrow(match(kind, direction))
  end

  LG.setBlendMode("alpha")
  LG.pop()
end

return {
  update = function(dt) tweens.update(dt) end,
  render = render
}
